@@ -16,18 +16,17 @@ from .context import register_backends
 from .cron.scheduler import CronScheduler
 from .event.processor import EventProcessor
 from .logger import Logger
-from .message.event import Event, StopEvent
-from .message.event.application import ApplicationStartedEvent, ApplicationStoppedEvent
+from .message.event import Event
+from .message.event.application import ApplicationStartedEvent
 from .message.request import Request
 from .message.response import Response
 from .utils import set_thread_name
 
 
 __author__ = 'Fabio Manganiello <info@fabiomanganiello.com>'
-__version__ = '0.13.8'
+__version__ = '0.13.9'
 
 logger = logging.getLogger('platypush')
-logger.setLevel(logging.INFO)
 
 
 class Daemon:
@@ -76,11 +75,14 @@ class Daemon:
         Config.init(self.config_file)
         logging.basicConfig(**Config.get('logging'))
 
+        redis_conf = Config.get('backend.redis') or {}
+        self.bus = RedisBus(on_message=self.on_message(), **redis_conf.get('redis_args', {}))
         self.no_capture_stdout = no_capture_stdout
         self.no_capture_stderr = no_capture_stderr
         self.event_processor = EventProcessor()
         self.requests_to_process = requests_to_process
         self.processed_requests = 0
+        self.cron_scheduler = None
 
     @classmethod
     def build_from_cmdline(cls, args):
@@ -136,9 +138,6 @@ class Daemon:
                     self.stop_app()
             elif isinstance(msg, Response):
                 logger.info('Received response: {}'.format(msg))
-            elif isinstance(msg, StopEvent) and msg.targets_me():
-                logger.info('Received STOP event: {}'.format(msg))
-                self.stop_app()
             elif isinstance(msg, Event):
                 if not msg.disable_logging:
                     logger.info('Received event: {}'.format(msg))
@@ -150,9 +149,12 @@ class Daemon:
         """ Stops the backends and the bus """
         for backend in self.backends.values():
             backend.stop()
-        self.bus.stop()
 
-    def start(self):
+        self.bus.stop()
+        if self.cron_scheduler:
+            self.cron_scheduler.stop()
+
+    def run(self):
         """ Start the daemon """
         if not self.no_capture_stdout:
             sys.stdout = Logger(logger.info)
@@ -160,12 +162,7 @@ class Daemon:
             sys.stderr = Logger(logger.warning)
 
         set_thread_name('platypush')
-
-        print('---- Starting platypush v.{}'.format(__version__))
-
-        redis_conf = Config.get('backend.redis') or {}
-        self.bus = RedisBus(on_message=self.on_message(),
-                            **redis_conf.get('redis_args', {}))
+        logger.info('---- Starting platypush v.{}'.format(__version__))
 
         # Initialize the backends and link them to the bus
         self.backends = register_backends(bus=self.bus, global_scope=True)
@@ -176,7 +173,8 @@ class Daemon:
 
         # Start the cron scheduler
         if Config.get_cronjobs():
-            CronScheduler(jobs=Config.get_cronjobs()).start()
+            self.cron_scheduler = CronScheduler(jobs=Config.get_cronjobs())
+            self.cron_scheduler.start()
 
         self.bus.post(ApplicationStartedEvent())
 
@@ -186,17 +184,17 @@ class Daemon:
         except KeyboardInterrupt:
             logger.info('SIGINT received, terminating application')
         finally:
-            self.bus.post(ApplicationStoppedEvent())
             self.stop_app()
 
+        sys.exit(0)
+
 
 def main():
     """
     Platypush daemon main
     """
-
     app = Daemon.build_from_cmdline(sys.argv[1:])
-    app.start()
+    app.run()
 
 
 # vim:sw=4:ts=4:et:
