@@ -12,36 +12,136 @@ from platypush.utils import get_ssl_server_context, set_thread_name
 
 class HttpBackend(Backend):
     """
-    The HTTP backend is a general-purpose web server that you can leverage:
+    The HTTP backend is a general-purpose web server.
 
-        * To execute Platypush commands via HTTP calls. Example::
+    Example configuration:
 
-            curl -XPOST -H 'Content-Type: application/json' -H "X-Token: your_token" \\
-                -d '{
-                    "type":"request",
-                    "target":"nodename",
-                    "action":"tts.say",
-                    "args": {"phrase":"This is a test"}
-                }' \\
-                http://localhost:8008/execute
+      .. code-block:: yaml
+
+        backend.http:
+            # Default HTTP listen port
+            port: 8008
+            # Default websocket port
+            websocket_port: 8009
+            # External folders that will be exposed over `/resources/<name>`
+            resource_dirs:
+                photos: /mnt/hd/photos
+                videos: /mnt/hd/videos
+                music: /mnt/hd/music
+
+    You can leverage this backend:
+
+        * To execute Platypush commands via HTTP calls. In order to do so:
+
+            * Register a user to Platypush through the web panel (usually served on ``http://host:8008/``).
+
+            * Generate a token for your user, either through the web panel (Settings -> Generate Token) or via API:
+
+                .. code-block:: shell
+
+                    curl -XPOST -H 'Content-Type: application/json' -d '
+                      {
+                        "username": "$YOUR_USER",
+                        "password": "$YOUR_PASSWORD"
+                      }' http://host:8008/auth
+
+            * Execute actions through the ``/execute`` endpoint:
+
+                .. code-block:: shell
+
+                    curl -XPOST -H 'Content-Type: application/json' -H "Authorization: Bearer $YOUR_TOKEN" -d '
+                      {
+                        "type": "request",
+                        "action": "tts.say",
+                        "args": {
+                          "text": "This is a test"
+                        }
+                      }' http://host:8008/execute
 
         * To interact with your system (and control plugins and backends) through the Platypush web panel,
-          by default available on your web root document. Any plugin that you have configured and available as a panel
-          plugin will appear on the web panel as well as a tab.
+          by default available on ``http://host:8008/``. Any configured plugin that has an available panel
+          plugin will be automatically added to the web panel.
 
-        * To display a fullscreen dashboard with your configured widgets, by default available under ``/dashboard``
+        * To display a fullscreen dashboard with custom widgets.
 
-        * To stream media over HTTP through the ``/media`` endpoint
+            * Widgets are available as Vue.js components under ``platypush/backend/http/webapp/src/components/widgets``.
+
+            * Explore their options (some may require some plugins or backends to be configured in order to work) and
+              create a new dashboard template under ``~/.config/platypush/dashboards``- e.g. ``main.xml``:
+
+                .. code-block:: xml
+
+                    <Dashboard>
+                        <!-- Display the following widgets on the same row. Each row consists of 12 columns.
+                             You can specify the width of each widget either through class name (e.g. col-6 means
+                             6 columns out of 12, e.g. half the size of the row) or inline style
+                             (e.g. `style="width: 50%"`). -->
+                        <Row>
+                            <!-- Show a calendar widget with the upcoming events. It requires the `calendar` plugin to
+                                 be enabled and configured. -->
+                            <Calendar class="col-6" />
+
+                            <!-- Show the current track and other playback info. It requires `music.mpd` plugin or any
+                                 other music plugin enabled. -->
+                            <Music class="col-3" />
+
+                            <!-- Show current date, time and weather. It requires a `weather` plugin or backend enabled -->
+                            <DateTimeWeather class="col-3" />
+                        </Row>
+
+                        <!-- Display the following widgets on a second row -->
+                        <Row>
+                            <!-- Show a carousel of images from a local folder. For security reasons, the folder must be
+                                 explicitly exposed as an HTTP resource through the backend `resource_dirs` attribute. -->
+                            <ImageCarousel class="col-6" img-dir="/mnt/hd/photos/carousel" />
+
+                            <!-- Show the news headlines parsed from a list of RSS feed and stored locally through the
+                                 `http.poll` backend -->
+                            <RssNews class="col-6" db="sqlite:////path/to/your/rss.db" />
+                        </Row>
+                    </Dashboard>
+
+            * The dashboard will be accessible under ``http://host:8008/dashboard/<name>``, where ``name=main`` if for
+              example you stored your template under ``~/.config/platypush/dashboards/main.xml``.
+
+        * To expose custom endpoints that can be called as web hooks by other applications and run some custom logic.
+          All you have to do in this case is to create a hook on a
+          :class:`platypush.message.event.http.hook.WebhookEvent` with the endpoint that you want to expose and store
+          it under e.g. ``~/.config/platypush/scripts/hooks.py``:
+
+            .. code-block:: python
+
+                from platypush.context import get_plugin
+                from platypush.event.hook import hook
+                from platypush.message.event.http.hook import WebhookEvent
+
+                hook_token = 'abcdefabcdef'
+
+                # Expose the hook under the /hook/lights_toggle endpoint
+                @hook(WebhookEvent, hook='lights_toggle')
+                def lights_toggle(event, **context):
+                    # Do any checks on the request
+                    assert event.headers.get('X-Token') == hook_token, 'Unauthorized'
+
+                    # Run some actions
+                    lights = get_plugin('light.hue')
+                    lights.toggle()
 
     Any plugin can register custom routes under ``platypush/backend/http/app/routes/plugins``.
     Any additional route is managed as a Flask blueprint template and the `.py`
     module can expose lists of routes to the main webapp through the
     ``__routes__`` object (a list of Flask blueprints).
 
-    Note that if you set up a main token, it will be required for any HTTP
-    interaction - either as ``X-Token`` HTTP header, on the query string
-    (attribute name: ``token``), as part of the JSON payload root (attribute
-    name: ``token``), or via HTTP basic auth (any username works).
+    Security: Access to the endpoints requires at least one user to be registered. Access to the endpoints is regulated
+    in the following ways (with the exception of event hooks, whose logic is up to the user):
+
+        * **Simple authentication** - i.e. registered username and password.
+        * **JWT token** provided either over as ``Authorization: Bearer`` header or ``GET`` ``?token=<TOKEN>``
+          parameter. A JWT token can be generated either through the web panel or over the ``/auth`` endpoint.
+        * **Global platform token**, usually configured on the root of the ``config.yaml`` as ``token: <VALUE>``.
+          It can provided either over on the ``X-Token`` header or as a ``GET`` ``?token=<TOKEN>`` parameter.
+        * **Session token**, generated upon login, it can be used to authenticate requests through the ``Cookie`` header
+          (cookie name: ``session_token``).
 
     Requires:
 
@@ -71,7 +171,8 @@ class HttpBackend(Backend):
 
     def __init__(self, port=_DEFAULT_HTTP_PORT,
                  websocket_port=_DEFAULT_WEBSOCKET_PORT,
-                 disable_websocket=False, dashboard=None, resource_dirs=None,
+                 bind_address='0.0.0.0',
+                 disable_websocket=False, resource_dirs=None,
                  ssl_cert=None, ssl_key=None, ssl_cafile=None, ssl_capath=None,
                  maps=None, run_externally=False, uwsgi_args=None, **kwargs):
         """
@@ -81,6 +182,9 @@ class HttpBackend(Backend):
         :param websocket_port: Listen port for the websocket server (default: 8009)
         :type websocket_port: int
 
+        :param bind_address: Address/interface to bind to (default: 0.0.0.0, accept connection from any IP)
+        :type bind_address: str
+
         :param disable_websocket: Disable the websocket interface (default: False)
         :type disable_websocket: bool
 
@@ -104,38 +208,6 @@ class HttpBackend(Backend):
             the value is the absolute path to expose.
         :type resource_dirs: dict[str, str]
 
-        :param dashboard: Set it if you want to use the dashboard service. It will contain the configuration for the
-            widgets to be used (look under ``platypush/backend/http/templates/widgets/`` for the available widgets).
-
-        Example configuration::
-
-            dashboard:
-                background_image: https://site/image.png
-                widgets:                # Each row of the dashboard will have 6 columns
-                    -
-                        widget: calendar           # Calendar widget
-                        columns: 6
-                    -
-                        widget: music              # Music widget
-                        columns: 3
-                    -
-                        widget: date-time-weather  # Date, time and weather widget
-                        columns: 3
-                    -
-                        widget: image-carousel     # Image carousel
-                        columns: 6
-                        # Absolute path (valid as long as it's a subdirectory of one of the available `resource_dirs`)
-                        images_path: ~/Dropbox/Photos/carousel
-                        refresh_seconds: 15
-                    -
-                        widget: rss-news           # RSS feeds widget
-                        # Requires backend.http.poll to be enabled with some RSS sources and write them to sqlite db
-                        columns: 6
-                        limit: 25
-                        db: "sqlite:////home/user/.local/share/platypush/feeds/rss.db"
-
-        :type dashboard: dict
-
         :param run_externally: If set, then the HTTP backend will not directly
             spawn the web server. Set this option if you plan to run the webapp
             in a separate web server (recommended), like uwsgi or uwsgi+nginx.
@@ -161,11 +233,12 @@ class HttpBackend(Backend):
 
         self.port = port
         self.websocket_port = websocket_port
-        self.dashboard = dashboard or {}
         self.maps = maps or {}
         self.server_proc = None
         self.disable_websocket = disable_websocket
         self.websocket_thread = None
+        self._websocket_loop = None
+        self.bind_address = bind_address
 
         if resource_dirs:
             self.resource_dirs = {name: os.path.abspath(
@@ -204,10 +277,24 @@ class HttpBackend(Backend):
         if self.server_proc:
             if isinstance(self.server_proc, subprocess.Popen):
                 self.server_proc.kill()
-                self.server_proc.wait()
+                self.server_proc.wait(timeout=10)
+                if self.server_proc.poll() is not None:
+                    self.logger.warning('HTTP server process still alive at termination')
+                else:
+                    self.logger.info('HTTP server process terminated')
             else:
                 self.server_proc.terminate()
-                self.server_proc.join()
+                self.server_proc.join(timeout=10)
+                if self.server_proc.is_alive():
+                    self.server_proc.kill()
+                if self.server_proc.is_alive():
+                    self.logger.warning('HTTP server process still alive at termination')
+                else:
+                    self.logger.info('HTTP server process terminated')
+
+        if self.websocket_thread and self.websocket_thread.is_alive() and self._websocket_loop:
+            self._websocket_loop.stop()
+            self.logger.info('HTTP websocket service terminated')
 
     def _acquire_websocket_lock(self, ws):
         try:
@@ -288,17 +375,17 @@ class HttpBackend(Backend):
         if self.ssl_context:
             websocket_args['ssl'] = self.ssl_context
 
-        loop = get_or_create_event_loop()
-        loop.run_until_complete(
-            websockets.serve(register_websocket, '0.0.0.0', self.websocket_port,
+        self._websocket_loop = get_or_create_event_loop()
+        self._websocket_loop.run_until_complete(
+            websockets.serve(register_websocket, self.bind_address, self.websocket_port,
                              **websocket_args))
-        loop.run_forever()
+        self._websocket_loop.run_forever()
 
     def _start_web_server(self):
         def proc():
             self.logger.info('Starting local web server on port {}'.format(self.port))
             kwargs = {
-                'host': '0.0.0.0',
+                'host': self.bind_address,
                 'port': self.port,
                 'use_reloader': False,
                 'debug': False,
