@@ -109,15 +109,22 @@ def send_request(action, wait_for_response=True, **kwargs):
 
 def _authenticate_token():
     token = Config.get('token')
+    user_manager = UserManager()
 
     if 'X-Token' in request.headers:
         user_token = request.headers['X-Token']
+    elif 'Authorization' in request.headers and request.headers['Authorization'].startswith('Bearer '):
+        user_token = request.headers['Authorization'][len('Bearer '):]
     elif 'token' in request.args:
         user_token = request.args.get('token')
     else:
         return False
 
-    return token and user_token == token
+    try:
+        user_manager.validate_jwt_token(user_token)
+        return True
+    except:
+        return token and user_token == token
 
 
 def _authenticate_http():
@@ -177,7 +184,6 @@ def authenticate(redirect_page='', skip_auth_methods=None, check_csrf_token=Fals
         def wrapper(*args, **kwargs):
             user_manager = UserManager()
             n_users = user_manager.get_user_count()
-            token = Config.get('token')
             skip_methods = skip_auth_methods or []
 
             # User/pass HTTP authentication
@@ -189,7 +195,7 @@ def authenticate(redirect_page='', skip_auth_methods=None, check_csrf_token=Fals
 
             # Token-based authentication
             token_auth_ok = True
-            if token and 'token' not in skip_methods:
+            if 'token' not in skip_methods:
                 token_auth_ok = _authenticate_token()
                 if token_auth_ok:
                     return f(*args, **kwargs)
@@ -210,7 +216,7 @@ def authenticate(redirect_page='', skip_auth_methods=None, check_csrf_token=Fals
                     return abort(403, 'Invalid or missing csrf_token')
 
             if n_users == 0 and 'session' not in skip_methods:
-                return redirect('/register?redirect=' + redirect_page, 307)
+                return redirect('/register?redirect=' + (redirect_page or request.url), 307)
 
             if ('http' not in skip_methods and http_auth_ok) or \
                     ('token' not in skip_methods and token_auth_ok) or \
