@@ -1,7 +1,10 @@
-from platypush.plugins import Plugin, action
+from abc import ABC
+
+from platypush.plugins import action
+from platypush.plugins.switch import SwitchPlugin
 
 
-class LightPlugin(Plugin):
+class LightPlugin(SwitchPlugin, ABC):
     """
     Abstract plugin to interface your logic with lights/bulbs.
     """
@@ -21,10 +24,5 @@ class LightPlugin(Plugin):
         """ Toggle the light status (on/off) """
         raise NotImplementedError()
 
-    @action
-    def status(self, *args, **kwargs):
-        """ Get the light status """
-        raise NotImplementedError()
-
 
 # vim:sw=4:ts=4:et:
