@@ -156,7 +156,6 @@ class MediaChromecastPlugin(MediaPlugin):
         :type callback: func
         """
 
-        import pychromecast
         self.chromecasts.update({
             cast.device.friendly_name: cast
             for cast in self._get_chromecasts(tries=tries, retry_wait=retry_wait,
@@ -173,8 +172,8 @@ class MediaChromecastPlugin(MediaPlugin):
             'manufacturer': cc.device.manufacturer,
             'model_name': cc.model_name,
             'uuid': str(cc.device.uuid),
-            'address': cc.host,
-            'port': cc.port,
+            'address': cc.host if hasattr(cc, 'host') else cc.uri.split(':')[0],
+            'port': cc.port if hasattr(cc, 'port') else int(cc.uri.split(':')[1]),
 
             'status': ({
                 'app': {
@@ -296,12 +295,6 @@ class MediaChromecastPlugin(MediaPlugin):
 
         resource = self._get_resource(resource)
 
-        if resource.startswith('magnet:?'):
-            player_args = {'chromecast': chromecast}
-            return get_plugin('media.webtorrent').play(resource,
-                                                       player='chromecast',
-                                                       **player_args)
-
         if not content_type:
             content_type = get_mime_type(resource)
 
@@ -390,7 +383,7 @@ class MediaChromecastPlugin(MediaPlugin):
         return self.forward(chromecast=chromecast, offset=position)
 
     @action
-    def back(self, chromecast=None, offset=60):
+    def back(self, chromecast=None, offset=30):
         cast = self.get_chromecast(chromecast or self.chromecast)
         mc = cast.media_controller
         if mc.status.current_time:
@@ -400,7 +393,7 @@ class MediaChromecastPlugin(MediaPlugin):
         return self.status(chromecast=chromecast)
 
     @action
-    def forward(self, chromecast=None, offset=60):
+    def forward(self, chromecast=None, offset=30):
         cast = self.get_chromecast(chromecast or self.chromecast)
         mc = cast.media_controller
         if mc.status.current_time:
