@@ -1,8 +1,10 @@
 import json
 import os
+import pathlib
 import random
 import shutil
 import threading
+from contextlib import contextmanager
 from datetime import datetime
 from typing import List, Dict, Any, Union, Optional, Tuple, Iterable
 
@@ -57,36 +59,76 @@ class TensorflowPlugin(Plugin):
 
     def __init__(self, workdir: Optional[str] = None, **kwargs):
         """
-        :param workdir: Working directory for TensorFlow, where models will be stored
+        :param workdir: Working directory for TensorFlow, where models will be stored and looked up by default
             (default: PLATYPUSH_WORKDIR/tensorflow).
         """
         super().__init__(**kwargs)
         self.models: Dict[str, Model] = {}
+        self._models_lock = threading.RLock()
         self._model_locks: Dict[str, threading.RLock()] = {}
         self._work_dir = os.path.abspath(os.path.expanduser(workdir)) if workdir else \
             os.path.join(Config.get('workdir'), 'tensorflow')
 
         self._models_dir = os.path.join(self._work_dir, 'models')
-        os.makedirs(self._models_dir, mode=0o755, exist_ok=True)
+        pathlib.Path(self._models_dir).mkdir(mode=0o755, exist_ok=True, parents=True)
 
-    def _load_model(self, name: str, reload: bool = False) -> Model:
-        if name in self.models and not reload:
-            return self.models[name]
+    @contextmanager
+    def _lock_model(self, model_name: str):
+        with self._models_lock:
+            if model_name not in self._model_locks:
+                self._model_locks[model_name] = threading.RLock()
 
-        model_dir = os.path.join(self._models_dir, name)
-        assert os.path.isdir(model_dir), 'The model {} does not exist'.format(name)
-        model = load_model(model_dir)
+        try:
+            success = self._model_locks[model_name].acquire(blocking=True, timeout=30.)
+            assert success, 'Unable to acquire the model lock'
+            yield
+        finally:
+            # noinspection PyBroadException
+            try:
+                self._model_locks[model_name].release()
+            except:
+                pass
+
+    def _load_model(self, model_name: str, reload: bool = False) -> Model:
+        if model_name in self.models and not reload:
+            return self.models[model_name]
+
+        model = None
+        model_dir = None
+
+        if os.path.isdir(os.path.join(self._models_dir, model_name)):
+            model_dir = os.path.join(self._models_dir, model_name)
+            model = load_model(model_dir)
+        else:
+            model_name = os.path.abspath(os.path.expanduser(model_name))
+            if model_name in self.models and not reload:
+                return self.models[model_name]
+
+            if os.path.isfile(model_name):
+                model_dir = str(pathlib.Path(model_name).parent)
+                model = load_model(model_name)
+            elif os.path.isdir(model_name):
+                model_dir = model_name
+                model = load_model(model_dir)
+
+        assert model, 'Could not find model: {}'.format(model_name)
         model.input_labels = []
         model.output_labels = []
-
         labels_file = os.path.join(model_dir, 'labels.json')
+
         if os.path.isfile(labels_file):
             with open(labels_file, 'r') as f:
                 labels = json.load(f)
-            if 'input' in labels:
-                model.input_labels = labels['input']
-            if 'output' in labels:
-                model.output_labels = labels['output']
+            if isinstance(labels, dict):
+                if 'input' in labels:
+                    model.input_labels = labels['input']
+                if 'output' in labels:
+                    model.output_labels = labels['output']
+            elif hasattr(labels, '__iter__'):
+                model.output_labels = labels
+
+        with self._lock_model(model_name):
+            self.models[model_name] = model
 
         return model
 
@@ -142,7 +184,8 @@ class TensorflowPlugin(Plugin):
         """
         (Re)-load a model from the file system.
 
-        :param model: Name of the model. Must be a folder name stored under ``<workdir>/models``.
+        :param model: Name of the model. It can be a folder name stored under ``<workdir>/models``, or an absolute path
+            to a model directory or file (Tensorflow directories, Protobuf models and HDF5 files are supported).
         :param reload: If ``True``, the model will be reloaded from the filesystem even if it's been already
             loaded, otherwise the model currently in memory will be kept (default: ``False``).
         :return: The model configuration.
@@ -157,8 +200,9 @@ class TensorflowPlugin(Plugin):
 
         :param model: Name of the model.
         """
-        assert model in self.models, 'The model {} is not loaded'.format(model)
-        del self.models[model]
+        with self._lock_model(model):
+            assert model in self.models, 'The model {} is not loaded'.format(model)
+            del self.models[model]
 
     @action
     def remove(self, model: str) -> None:
@@ -168,8 +212,9 @@ class TensorflowPlugin(Plugin):
 
         :param model: Name of the model.
         """
-        if model in self.models:
-            del self.models[model]
+        with self._lock_model(model):
+            if model in self.models:
+                del self.models[model]
 
         model_dir = os.path.join(self._models_dir, model)
         if os.path.isdir(model_dir):
@@ -388,7 +433,9 @@ class TensorflowPlugin(Plugin):
 
         model.input_labels = input_names or []
         model.output_labels = output_names or []
-        self.models[name] = model
+
+        with self._lock_model(name):
+            self.models[name] = model
         return model.get_config()
 
     @action
@@ -520,7 +567,8 @@ class TensorflowPlugin(Plugin):
             **kwargs
         )
 
-        self.models[name] = model
+        with self._lock_model(name):
+            self.models[name] = model
         return model.get_config()
 
     @staticmethod
@@ -551,10 +599,7 @@ class TensorflowPlugin(Plugin):
             model.name, size)
 
         colors = input_shape[3:]
-        assert colors, ('The model {} requires a tensor with at least 3 inputs in order to process images: ' +
-                        '[WIDTH, HEIGHT, COLORS]').format(model.name)
-
-        if colors[0] == 1:
+        if len(colors) == 0 or colors[0] == 1:
             color_mode = 'grayscale'
         elif colors[0] == 3:
             color_mode = 'rgb'
@@ -565,7 +610,11 @@ class TensorflowPlugin(Plugin):
                                  'Found: {}'.format(colors[0]))
 
         img = image.load_img(image_file, target_size=size, color_mode=color_mode)
-        return image.img_to_array(img)
+        data = image.img_to_array(img)
+        if data.shape[-1] == 1:
+            # Squeeze extra color channels
+            data = np.squeeze(data)
+        return data
 
     @classmethod
     def _get_dir(cls, directory: str, model: Model) -> Dict[str, Iterable]:
@@ -585,9 +634,28 @@ class TensorflowPlugin(Plugin):
 
         return ret
 
+    @classmethod
+    def _get_outputs(cls, data: Union[str, np.ndarray, Iterable], model: Model) -> np.ndarray:
+        if isinstance(data, str):
+            if model.output_labels:
+                label_index = model.output_labels.index(data)
+                if label_index >= 0:
+                    return np.array([1 if i == label_index else 0 for i in range(len(model.output_labels))])
+
+            return np.array([data])
+
+        if len(data) > 0 and isinstance(data[0], str):
+            return np.array([cls._get_outputs(item, model) for item in data])
+
+        return data
+
     @classmethod
     def _get_data(cls, data: Union[str, np.ndarray, Iterable, Dict[str, Union[Iterable, np.ndarray]]], model: Model) \
             -> Union[np.ndarray, Iterable, Dict[str, Union[Iterable, np.ndarray]]]:
+        if isinstance(data, List) or isinstance(data, Tuple):
+            if len(data) and isinstance(data[0], str):
+                return np.array([cls._get_data(item, model) for item in data])
+
         if not isinstance(data, str):
             return data
 
@@ -619,6 +687,8 @@ class TensorflowPlugin(Plugin):
         elif os.path.isdir(data_file):
             return cls._get_dir(data_file, model)
 
+        return data
+
     @classmethod
     def _get_dataset(cls,
                      inputs: Union[str, np.ndarray, Iterable, Dict[str, Union[Iterable, np.ndarray]]],
@@ -628,12 +698,13 @@ class TensorflowPlugin(Plugin):
                      Optional[Union[np.ndarray, Iterable, Dict[str, Union[Iterable, np.ndarray]]]]]:
         inputs = cls._get_data(inputs, model)
         if outputs:
-            outputs = cls._get_data(inputs, model)
+            outputs = cls._get_outputs(outputs, model)
         elif isinstance(inputs, dict) and model.output_labels:
             pairs = []
             for i, label in enumerate(model.output_labels):
                 data = inputs.get(label, [])
-                pairs.extend([(d, i) for d in data])
+                pairs.extend([(d, tuple(1 if i == j else 0 for j, _ in enumerate(model.output_labels)))
+                              for d in data])
 
             random.shuffle(pairs)
             inputs = np.asarray([p[0] for p in pairs])
@@ -664,7 +735,8 @@ class TensorflowPlugin(Plugin):
         """
         Trains a model on a dataset for a fixed number of epochs.
 
-        :param model: Name of the existing model to be trained.
+        :param model: Name of the model. It can be a folder name stored under ``<workdir>/models``, or an absolute path
+            to a model directory or file (Tensorflow directories, Protobuf models and HDF5 files are supported).
         :param inputs: Input data. It can be:
 
           - A numpy array (or array-like), or a list of arrays in case the model has multiple inputs.
@@ -801,7 +873,7 @@ class TensorflowPlugin(Plugin):
             use_multiprocessing=use_multiprocessing,
         )
 
-        return TensorflowTrainResponse(model=model, epochs=ret.epoch, history=ret.history)
+        return TensorflowTrainResponse(model=model, model_name=name, epochs=ret.epoch, history=ret.history)
 
     @action
     def evaluate(self,
@@ -818,7 +890,8 @@ class TensorflowPlugin(Plugin):
         """
         Returns the loss value and metrics values for the model in test model.
 
-        :param model: Name of the existing model to be trained.
+        :param model: Name of the model. It can be a folder name stored under ``<workdir>/models``, or an absolute path
+            to a model directory or file (Tensorflow directories, Protobuf models and HDF5 files are supported).
         :param inputs: Input data. It can be:
 
           - A numpy array (or array-like), or a list of arrays in case the model has multiple inputs.
@@ -922,7 +995,8 @@ class TensorflowPlugin(Plugin):
         """
         Generates output predictions for the input samples.
 
-        :param model: Name of the existing model to be trained.
+        :param model: Name of the model. It can be a folder name stored under ``<workdir>/models``, or an absolute path
+            to a model directory or file (Tensorflow directories, Protobuf models and HDF5 files are supported).
         :param inputs: Input data. It can be:
 
           - A numpy array (or array-like), or a list of arrays in case the model has multiple inputs.
@@ -963,21 +1037,21 @@ class TensorflowPlugin(Plugin):
 
         :return: :class:`platypush.message.response.tensorflow.TensorflowPredictResponse`. Format:
 
-            - For regression models with no output labels specified: ``values`` will contain the output vector:
+            - For regression models with no output labels specified: ``outputs`` will contain the output vector:
 
                 .. code-block:: json
 
                   {
-                      "values": [[3.1415]]
+                      "outputs": [[3.1415]]
                   }
 
-            - For regression models with output labels specified: ``values`` will be a list of ``{label -> value}``
+            - For regression models with output labels specified: ``outputs`` will be a list of ``{label -> value}``
               maps:
 
                 .. code-block:: json
 
                   {
-                      "values": [
+                      "outputs": [
                           {
                               "x": 42.0,
                               "y": 43.0
@@ -985,17 +1059,17 @@ class TensorflowPlugin(Plugin):
                       ]
                   }
 
-            - For neural networks: ``values`` will contain the list of the output vector like in the case of
-              regression, and ``labels`` will store the list of ``argmax`` (i.e. the index of the output unit with the
+            - For neural networks: ``outputs`` will contain the list of the output vector like in the case of
+              regression, and ``predictions`` will store the list of ``argmax`` (i.e. the index of the output unit with the
               highest value) or their labels, if the model has output labels:
 
                 .. code-block:: json
 
                   {
-                      "labels": [
+                      "predictions": [
                           "positive"
                       ],
-                      "values": [
+                      "outputs": [
                           {
                               "positive": 0.998,
                               "negative": 0.002
@@ -1023,7 +1097,8 @@ class TensorflowPlugin(Plugin):
             use_multiprocessing=use_multiprocessing
         )
 
-        return TensorflowPredictResponse(model=model, prediction=ret, output_labels=model.output_labels)
+        return TensorflowPredictResponse(model=model, model_name=name, prediction=ret,
+                                         output_labels=model.output_labels)
 
     @action
     def save(self, model: str, overwrite: bool = True, **opts) -> None:
@@ -1035,22 +1110,39 @@ class TensorflowPlugin(Plugin):
         :param overwrite: Overwrite the model files if they already exist.
         :param opts: Extra options to be passed to ``Model.save()``.
         """
-        assert model in self.models, 'No such model in memory: {}'.format(model)
-        model_dir = os.path.join(self._models_dir, model)
-        model = self.models[model]
-        os.makedirs(model_dir, exist_ok=True)
-        labels = {}
-        labels_file = os.path.join(model_dir, 'labels.json')
+        model_name = model
+        model_dir = None
 
-        if model.input_labels:
-            labels['input'] = model.input_labels
-        if model.output_labels:
-            labels['output'] = model.output_labels
-        if labels:
-            with open(labels_file, 'w') as f:
-                json.dump(labels, f)
+        if os.path.isdir(os.path.join(self._models_dir, model_name)) or model_name in self.models:
+            model_dir = os.path.join(self._models_dir, model_name)
+        else:
+            model_file = os.path.abspath(os.path.expanduser(model_name))
+            if os.path.isfile(model_file):
+                model_dir = str(pathlib.Path(model_file).parent)
+            elif os.path.isdir(model_file):
+                model_dir = model_file
 
-        model.save(model_dir, overwrite=overwrite, options=opts)
+        model = self.models.get(model_name, self.models.get(model_dir))
+        assert model, 'No such model loaded: {}'.format(model_name)
+        pathlib.Path(model_dir).mkdir(parents=True, exist_ok=True)
+
+        with self._lock_model(model_name):
+            labels = {}
+            labels_file = os.path.join(model_dir, 'labels.json')
+
+            if hasattr(model, 'input_labels') and model.input_labels:
+                labels['input'] = model.input_labels
+            if hasattr(model, 'output_labels') and model.output_labels:
+                if hasattr(labels, 'input'):
+                    labels['output'] = model.output_labels
+                else:
+                    labels = model.output_labels
+
+            if labels:
+                with open(labels_file, 'w') as f:
+                    json.dump(labels, f)
+
+            model.save(model_name if os.path.isfile(model_name) else model_dir, overwrite=overwrite, options=opts)
 
 
 # vim:sw=4:ts=4:et:
