@@ -86,10 +86,9 @@ class ZeroconfPlugin(Plugin):
         :param service: Service type (e.g. ``_http._tcp.local.``).
         :param timeout: Browser timeout in seconds (default: 5). Specify None for no timeout - in such case the
             discovery will loop forever and generate events upon service changes.
-        :return: A ``service_type -> [service_names]`` mapping. Example::
+        :return: A ``service_type -> [service_names]`` mapping. Example:
 
-
-        .. code-block:: json
+          .. code-block:: json
 
             {
                 "host1._platypush-http._tcp.local.": {
@@ -109,8 +108,7 @@ class ZeroconfPlugin(Plugin):
                         "server": "host1._platypush-http._tcp.local.",
                         "weight": 0
                     }
-                },
-                ...
+                }
             }
 
         """
@@ -128,7 +126,7 @@ class ZeroconfPlugin(Plugin):
             while timeout and time.time() - discovery_start < timeout:
                 to = discovery_start + timeout - time.time() if timeout else None
                 try:
-                    evt: ZeroconfEvent = evt_queue.get(block=True, timeout=to)
+                    evt = evt_queue.get(block=True, timeout=to)
                     if isinstance(evt, ZeroconfServiceAddedEvent) or isinstance(evt, ZeroconfServiceUpdatedEvent):
                         services[evt.service_name] = {
                             'type': evt.service_type,
