@@ -4,12 +4,13 @@ import importlib
 import inspect
 import logging
 import os
+import pathlib
 import re
 import signal
 import socket
 import ssl
 import urllib.request
-from typing import Optional
+from typing import Optional, Tuple
 
 logger = logging.getLogger('utils')
 
@@ -274,7 +275,16 @@ def is_process_alive(pid):
 
 def get_ip_or_hostname():
     ip = socket.gethostbyname(socket.gethostname())
-    return socket.getfqdn() if ip.startswith('127.') else ip
+    if ip.startswith('127.'):
+        try:
+            sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+            sock.connect(('10.255.255.255', 1))
+            ip = sock.getsockname()[0]
+            sock.close()
+        except:
+            pass
+
+    return ip
 
 
 def get_mime_type(resource):
@@ -294,7 +304,7 @@ def get_mime_type(resource):
             raise RuntimeError('The installed magic version provides neither detect_from_filename nor from_file')
 
         if mime:
-            return mime.mime_type
+            return mime.mime_type if hasattr(mime, 'mime_type') else mime
 
 
 def camel_case_to_snake_case(string):
@@ -342,4 +352,88 @@ def run(action, *args, **kwargs):
     return response.output
 
 
+def generate_rsa_key_pair(key_file: Optional[str] = None, size: int = 2048) -> Tuple[str, str]:
+    """
+    Generate an RSA key pair.
+
+    :param key_file: Target file for the private key (the associated public key will be stored in ``<key_file>.pub``.
+        If no key file is specified then the public and private keys will be returned in ASCII format in a dictionary
+        with the following structure:
+
+            .. code-block:: json
+
+                {
+                    "private": "private key here",
+                    "public": "public key here"
+                }
+
+    :param size: Key size (default: 2048 bits).
+    :return: A tuple with the generated ``(priv_key_str, pub_key_str)``.
+    """
+    from cryptography.hazmat.primitives import serialization
+    from cryptography.hazmat.primitives.asymmetric import rsa
+    from cryptography.hazmat.backends import default_backend
+
+    public_exp = 65537
+    private_key = rsa.generate_private_key(
+        public_exponent=public_exp,
+        key_size=size,
+        backend=default_backend()
+    )
+
+    logger.info('Generating RSA {} key pair'.format(size))
+    private_key_str = private_key.private_bytes(
+        encoding=serialization.Encoding.PEM,
+        format=serialization.PrivateFormat.TraditionalOpenSSL,
+        encryption_algorithm=serialization.NoEncryption()
+    ).decode()
+
+    public_key_str = private_key.public_key().public_bytes(
+        encoding=serialization.Encoding.PEM,
+        format=serialization.PublicFormat.PKCS1,
+    ).decode()
+
+    if key_file:
+        logger.info('Saving private key to {}'.format(key_file))
+        with open(os.path.expanduser(key_file), 'w') as f1, \
+                open(os.path.expanduser(key_file) + '.pub', 'w') as f2:
+            f1.write(private_key_str)
+            f2.write(public_key_str)
+            os.chmod(key_file, 0o600)
+
+    return public_key_str, private_key_str
+
+
+def get_or_generate_jwt_rsa_key_pair():
+    from platypush.config import Config
+
+    key_dir = os.path.join(Config.get('workdir'), 'jwt')
+    priv_key_file = os.path.join(key_dir, 'id_rsa')
+    pub_key_file = priv_key_file + '.pub'
+
+    if os.path.isfile(priv_key_file) and os.path.isfile(pub_key_file):
+        with open(pub_key_file, 'r') as f1, \
+                open(priv_key_file, 'r') as f2:
+            return f1.read(), f2.read()
+
+    pathlib.Path(key_dir).mkdir(parents=True, exist_ok=True, mode=0o755)
+    return generate_rsa_key_pair(priv_key_file, size=2048)
+
+
+def get_enabled_plugins() -> dict:
+    from platypush.config import Config
+    from platypush.context import get_plugin
+
+    plugins = {}
+    for name, config in Config.get_plugins().items():
+        try:
+            plugin = get_plugin(name)
+            if plugin:
+                plugins[name] = plugin
+        except:
+            pass
+
+    return plugins
+
+
 # vim:sw=4:ts=4:et:
