@@ -1,10 +1,11 @@
-from .context import platypush
-
+import logging
 import unittest
 
+from .context import config_file
 from platypush.event.hook import EventCondition
 from platypush.message.event.ping import PingEvent
 
+
 class TestEventParse(unittest.TestCase):
     def setUp(self):
         self.condition = EventCondition.build({
@@ -13,6 +14,7 @@ class TestEventParse(unittest.TestCase):
         })
 
     def test_event_parse(self):
+        logging.info('Starting test on configuration file: {}'.format(config_file))
         message = "GARBAGE GARBAGE this is the answer: 42"
         event = PingEvent(message=message)
         result = event.matches_condition(self.condition)
@@ -30,4 +32,3 @@ if __name__ == '__main__':
     unittest.main()
 
 # vim:sw=4:ts=4:et:
-
